@@ -94,6 +94,7 @@ VLC_BOOTSTRAP_ARGS="\
     --disable-mpg123 \
     --enable-libdsm \
     --enable-libarchive \
+    --disable-libmpeg2 \
 "
 
 ###########################
@@ -618,6 +619,13 @@ BUILTINS="$BUILTINS NULL\n};\n"; \
 printf "/* Autogenerated from the list of modules */\n$DEFINITION\n$BUILTINS\n" > libvlc/jni/libvlcjni-modules.h
 rm ${REDEFINED_VLC_MODULES_DIR}/syms
 
+# Generating the .ver file like libvlc.so upstream
+VER_FILE="vlc/$VLC_BUILD_DIR/lib/.libs/libvlc.ver"
+echo "{ global:" > $VER_FILE
+cat vlc/lib/libvlc.sym libvlc/libvlcjni.sym | sed -e "s/\(.*\)/\1;/" >> $VER_FILE
+echo "__gmp_binvert_limb_table;" >> $VER_FILE # FIXME
+echo "local: *; };" >> $VER_FILE
+
 ###############################
 # NDK-Build for libvlcjni.so  #
 ###############################
@@ -650,7 +658,7 @@ $ANDROID_NDK/ndk-build -C libvlc \
     VLC_CONTRIB="$VLC_CONTRIB" \
     VLC_MODULES="$VLC_MODULES" \
     TARGET_CFLAGS="$EXTRA_CFLAGS" \
-    EXTRA_LDFLAGS="$EXTRA_LDFLAGS" \
+    EXTRA_LDFLAGS="$EXTRA_LDFLAGS -Wl,-soname -Wl,libvlc.so.5 -Wl,-version-script -Wl,$SRC_DIR/$VER_FILE" \
     LIBVLC_LIBS="$LIBVLC_LIBS" \
     LIBIOMX_LIBS="$LIBIOMX_LIBS" \
     LIBANW_LIBS="$LIBANW_LIBS" \
