@@ -76,12 +76,12 @@ checkfail()
 
 if [ ! -d "gradle/wrapper" ]; then
     echo "Downloading gradle"
-    GRADLE_VERSION=2.2.1
+    GRADLE_VERSION=2.6
     GRADLE_URL=http://services.gradle.org/distributions/gradle-${GRADLE_VERSION}-all.zip
     wget ${GRADLE_URL} 2>/dev/null || curl -O ${GRADLE_URL}
     checkfail "gradle: download failed"
 
-    unzip gradle-${GRADLE_VERSION}-all.zip
+    unzip -o gradle-${GRADLE_VERSION}-all.zip
     checkfail "gradle: unzip failed"
 
     cd gradle-${GRADLE_VERSION}
@@ -131,7 +131,7 @@ fi
 # Fetch VLC source #
 ####################
 
-TESTED_HASH=e5325d7
+TESTED_HASH=d568828
 if [ ! -d "vlc" ]; then
     echo "VLC source not found, cloning"
     git clone git://git.videolan.org/vlc.git vlc
