@@ -215,9 +215,16 @@ class AWindow implements IAWindowNativeHandler, IVLCVout {
         mSurfaces[ID_SUBTITLES] = null;
     }
 
-    private void setView(int id, SurfaceView view) {
+    private void ensureInitState() throws IllegalStateException {
         if (mSurfacesState.get() != SURFACE_STATE_INIT)
-            throw new IllegalStateException("Can't set view when already attached");
+            throw new IllegalStateException("Can't set view when already attached. " +
+                    "Current state: " + mSurfacesState.get() + ", " +
+                    "mSurfaces[ID_VIDEO]: " + mSurfaceHelpers[ID_VIDEO] + " / " + mSurfaces[ID_VIDEO] + ", " +
+                    "mSurfaces[ID_SUBTITLES]: " + mSurfaceHelpers[ID_SUBTITLES] + " / " + mSurfaces[ID_SUBTITLES]);
+    }
+
+    private void setView(int id, SurfaceView view) {
+        ensureInitState();
         if (view == null)
             throw new NullPointerException("view is null");
         final SurfaceHelper surfaceHelper = mSurfaceHelpers[id];
@@ -230,8 +237,7 @@ class AWindow implements IAWindowNativeHandler, IVLCVout {
     private void setView(int id, TextureView view) {
         if (!AndroidUtil.isICSOrLater())
             throw new IllegalArgumentException("TextureView not implemented in this android version");
-        if (mSurfacesState.get() != SURFACE_STATE_INIT)
-            throw new IllegalStateException("Can't set view when already attached");
+        ensureInitState();
         if (view == null)
             throw new NullPointerException("view is null");
         final SurfaceHelper surfaceHelper = mSurfaceHelpers[id];
@@ -242,8 +248,7 @@ class AWindow implements IAWindowNativeHandler, IVLCVout {
     }
 
     private void setSurface(int id, Surface surface, SurfaceHolder surfaceHolder) {
-        if (mSurfacesState.get() != SURFACE_STATE_INIT)
-            throw new IllegalStateException("Can't set surface when already attached");
+        ensureInitState();
         if (!surface.isValid() || surfaceHolder == null)
             throw new IllegalStateException("surface is not attached and holder is null");
         final SurfaceHelper surfaceHelper = mSurfaceHelpers[id];
@@ -270,6 +275,12 @@ class AWindow implements IAWindowNativeHandler, IVLCVout {
         setSurface(ID_VIDEO, videoSurface, surfaceHolder);
     }
 
+    @Override
+    @TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH)
+    public void setVideoSurface(SurfaceTexture videoSurfaceTexture) {
+        setSurface(ID_VIDEO, new Surface(videoSurfaceTexture), null);
+    }
+
     @Override
     @MainThread
     public void setSubtitlesView(SurfaceView subtitlesSurfaceView) {
@@ -287,6 +298,12 @@ class AWindow implements IAWindowNativeHandler, IVLCVout {
         setSurface(ID_SUBTITLES, subtitlesSurface, surfaceHolder);
     }
 
+    @Override
+    @TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH)
+    public void setSubtitlesSurface(SurfaceTexture subtitlesSurfaceTexture) {
+        setSurface(ID_SUBTITLES, new Surface(subtitlesSurfaceTexture), null);
+    }
+
     @Override
     @MainThread
     public void attachViews() {
