@@ -24,6 +24,7 @@ import android.net.Uri;
 import android.os.Environment;
 import android.os.Handler;
 import android.os.Message;
+import android.support.v4.util.ArrayMap;
 import android.text.TextUtils;
 import android.util.Log;
 
@@ -44,7 +45,6 @@ import java.io.IOException;
 import java.lang.Thread.State;
 import java.lang.ref.WeakReference;
 import java.util.ArrayList;
-import java.util.HashMap;
 import java.util.HashSet;
 import java.util.LinkedList;
 import java.util.List;
@@ -80,7 +80,9 @@ public class MediaLibrary {
                 "/media/audio/alarms",
                 "/media/audio/notifications",
                 "/media/audio/ringtones",
-                "/Android/data/" };
+                "/android/data",
+                "/android/media",
+        };
 
         FOLDER_BLACKLIST = new HashSet<String>();
         for (String item : folder_blacklist)
@@ -94,17 +96,17 @@ public class MediaLibrary {
         mItemListLock = new ReentrantReadWriteLock();
     }
 
-    public void loadMediaItems(boolean restart) {
+    public void scanMediaItems(boolean restart) {
         if (restart && isWorking()) {
             /* do a clean restart if a scan is ongoing */
             mRestart = true;
             isStopping = true;
         } else {
-            loadMediaItems();
+            scanMediaItems();
         }
     }
 
-    public void loadMediaItems() {
+    public void scanMediaItems() {
         if (mLoadingThread == null || mLoadingThread.getState() == State.TERMINATED) {
             isStopping = false;
             Util.actionScanStart();
@@ -113,6 +115,19 @@ public class MediaLibrary {
         }
     }
 
+    public void loadMedaItems(){
+        VLCApplication.runBackground(new Runnable() {
+            @Override
+            public void run() {
+                mItemListLock.writeLock().lock();
+                mItemList.clear();
+                mItemList.addAll(getStoredMedias(MediaDatabase.getInstance()).values());
+                mItemListLock.writeLock().unlock();
+                notifyMediaUpdated();
+            }
+        });
+    }
+
     public void stop() {
         isStopping = true;
     }
@@ -266,7 +281,7 @@ public class MediaLibrary {
             directories.addAll(mediaDirs);
 
             // get all existing media items
-            HashMap<String, MediaWrapper> existingMedias = mediaDatabase.getMedias();
+            ArrayMap<String, MediaWrapper> existingMedias = getStoredMedias(mediaDatabase);
 
             // list of all added files
             HashSet<String> addedLocations = new HashSet<String>();
@@ -343,7 +358,9 @@ public class MediaLibrary {
                     for (String dirPath : dirsToIgnore) {
                         if (path.startsWith(dirPath)) {
                             mediasToRemove.add(entry.getValue().getUri());
+                            mItemListLock.writeLock().lock();
                             mItemList.remove(existingMedias.get(path));
+                            mItemListLock.writeLock().unlock();
                             continue outloop;
                         }
                     }
@@ -395,11 +412,7 @@ public class MediaLibrary {
                     }
                 }
             } finally {
-                // update the video and audio activities
-                for (int i = 0; i < mUpdateHandler.size(); i++) {
-                    Handler h = mUpdateHandler.get(i);
-                    h.sendEmptyMessage(MEDIA_ITEMS_UPDATED);
-                }
+                notifyMediaUpdated();
 
                 // remove old files & folders from database if storage is mounted
                 if (!isStopping && Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
@@ -433,6 +446,18 @@ public class MediaLibrary {
         }
     }
 
+    private void notifyMediaUpdated() {
+        // update the video and audio activities
+        for (int i = 0; i < mUpdateHandler.size(); i++) {
+            Handler h = mUpdateHandler.get(i);
+            h.sendEmptyMessage(MEDIA_ITEMS_UPDATED);
+        }
+    }
+
+    private ArrayMap<String, MediaWrapper> getStoredMedias(MediaDatabase mediaDatabase) {
+        return mediaDatabase.getMedias();
+    }
+
     private Handler restartHandler = new RestartHandler(this);
 
     private static class RestartHandler extends WeakHandler<MediaLibrary> {
@@ -444,7 +469,7 @@ public class MediaLibrary {
         public void handleMessage(Message msg) {
             MediaLibrary owner = getOwner();
             if(owner == null) return;
-            owner.loadMediaItems();
+            owner.scanMediaItems();
         }
     }
 
