@@ -1,7 +1,7 @@
 /*****************************************************************************
- * AudioService.java
+ * PlaybackService.java
  *****************************************************************************
- * Copyright © 2011-2013 VLC authors and VideoLAN
+ * Copyright © 2011-2015 VLC authors and VideoLAN
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -72,7 +72,6 @@ import org.videolan.vlc.util.Util;
 import org.videolan.vlc.util.VLCInstance;
 import org.videolan.vlc.util.VLCOptions;
 import org.videolan.vlc.util.WeakHandler;
-import org.videolan.vlc.widget.VLCAppWidgetProvider;
 
 import java.io.File;
 import java.net.URI;
@@ -91,7 +90,6 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
 
     private static final int SHOW_PROGRESS = 0;
     private static final int SHOW_TOAST = 1;
-    public static final String START_FROM_NOTIFICATION = "from_notification";
     public static final String ACTION_REMOTE_GENERIC = "org.videolan.vlc.remote.";
     public static final String ACTION_REMOTE_BACKWARD = "org.videolan.vlc.remote.Backward";
     public static final String ACTION_REMOTE_PLAY = "org.videolan.vlc.remote.Play";
@@ -147,8 +145,12 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     private int mNextIndex; // Set to -1 if no next media
 
     // Playback management
+
+    public static final int REPEAT_NONE = 0;
+    public static final int REPEAT_ONE = 1;
+    public static final int REPEAT_ALL = 2;
     private boolean mShuffling = false;
-    private RepeatType mRepeating = RepeatType.None;
+    private int mRepeating = REPEAT_NONE;
     private Random mRandom = null; // Used in shuffling process
 
     private boolean mHasAudioFocus = false;
@@ -178,13 +180,9 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                 mp.setAudioOutputDevice("hdmi");
         } else
             mIsAudioTrack = false;
-        return mp;
-    }
+        mp.getVLCVout().addCallback(this);
 
-    public static enum RepeatType {
-        None,
-        Once,
-        All
+        return mp;
     }
 
     private static boolean readPhoneState() {
@@ -196,7 +194,6 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         super.onCreate();
 
         mMediaPlayer = newMediaPlayer();
-        mMediaPlayer.getVLCVout().addCallback(this);
 
         if (!VLCInstance.testCompatibleCPU(this)) {
             stopSelf();
@@ -504,10 +501,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
              * Launch the activity if needed
              */
             if (action.startsWith(ACTION_REMOTE_GENERIC) && !mMediaPlayer.isPlaying() && !hasCurrentMedia()) {
-                Intent iVlc = new Intent(context, MainActivity.class);
-                iVlc.putExtra(START_FROM_NOTIFICATION, true);
-                iVlc.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_SINGLE_TOP);
-                context.startActivity(iVlc);
+                context.startActivity(getPackageManager().getLaunchIntentForPackage(getPackageName()));
             }
 
             /*
@@ -622,7 +616,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                          */
                         if (m != null && m.getLength() == 0 && length > 0) {
                             dbManager.updateMedia(mw.getUri(),
-                                    MediaDatabase.mediaColumn.MEDIA_LENGTH, length);
+                                    MediaDatabase.INDEX_MEDIA_LENGTH, length);
                         }
                     }
 
@@ -651,7 +645,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                     changeAudioFocus(false);
                     break;
                 case MediaPlayer.Event.EndReached:
-                    Log.i(TAG, "MediaPlayerEndReached");
+                    Log.i(TAG, "MediaPlayer.Event.EndReached");
                     executeUpdate();
                     executeUpdateProgress();
                     determinePrevAndNextIndices(true);
@@ -749,25 +743,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         }
     };
 
-    private void setVideoTrackEnabled(boolean enabled) {
-        if (!enabled) {
-            mMediaPlayer.setVideoTrack(-1);
-        } else {
-            final MediaPlayer.TrackDescription tracks[] = mMediaPlayer.getVideoTracks();
-
-            if (tracks != null) {
-                for (MediaPlayer.TrackDescription track : tracks) {
-                    if (track.id != -1) {
-                        mMediaPlayer.setVideoTrack(track.id);
-                        break;
-                    }
-                }
-            }
-        }
-    }
-
-
-    private boolean canSwitchToVideo() {
+    public boolean canSwitchToVideo() {
         return hasCurrentMedia() && mMediaPlayer.getVideoTracksCount() > 0;
     }
 
@@ -775,13 +751,10 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         if (!canSwitchToVideo() || !mMediaPlayer.isPlaying())
             return false;
         if (mMediaPlayer.getVLCVout().areViewsAttached()) {
-            setVideoTrackEnabled(true);
             hideNotification(false);
             return true;
-        } else {
-            setVideoTrackEnabled(false);
+        } else
             return false;
-        }
     }
 
     @MainThread
@@ -899,10 +872,10 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                 pendingIntent = PendingIntent.getBroadcast(this, 0, notificationIntent, PendingIntent.FLAG_UPDATE_CURRENT);
             } else {
                 /* Resume AudioPlayerActivity */
-                final Intent notificationIntent = new Intent(PlaybackService.this, MainActivity.class);
+
+                final Intent notificationIntent = getPackageManager().getLaunchIntentForPackage(getPackageName());
                 notificationIntent.setAction(AudioPlayerContainerActivity.ACTION_SHOW_PLAYER);
                 notificationIntent.addCategory(Intent.CATEGORY_LAUNCHER);
-                notificationIntent.putExtra(START_FROM_NOTIFICATION, true);
                 pendingIntent = PendingIntent.getActivity(this, 0, notificationIntent, PendingIntent.FLAG_UPDATE_CURRENT);
             }
 
@@ -993,6 +966,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
 
     @MainThread
     public void pause() {
+        savePosition();
         mHandler.removeMessages(SHOW_PROGRESS);
         // hideNotification(); <-- see event handler
         mMediaPlayer.pause();
@@ -1055,7 +1029,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
             mShuffling &= size > 2;
 
             // Repeating once doesn't change the index
-            if (mRepeating == RepeatType.Once) {
+            if (mRepeating == REPEAT_ONE) {
                 mPrevIndex = mNextIndex = mCurrentIndex;
             } else {
 
@@ -1065,7 +1039,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                     // If we've played all songs already in shuffle, then either
                     // reshuffle or stop (depending on RepeatType).
                     if(mPrevious.size() + 1 == size) {
-                        if(mRepeating == RepeatType.None) {
+                        if(mRepeating == REPEAT_NONE) {
                             mNextIndex = -1;
                             return;
                         } else {
@@ -1087,7 +1061,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                     if(mCurrentIndex + 1 < size)
                         mNextIndex = mCurrentIndex + 1;
                     else {
-                        if(mRepeating == RepeatType.None) {
+                        if(mRepeating == REPEAT_NONE) {
                             mNextIndex = -1;
                         } else {
                             mNextIndex = 0;
@@ -1154,40 +1128,44 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
 
     private void onMediaListChanged() {
         saveMediaList();
-        executeUpdate();
         determinePrevAndNextIndices();
+        executeUpdate();
     }
 
     @MainThread
     public void next() {
-        mPrevious.push(mCurrentIndex);
-        mCurrentIndex = mNextIndex;
-
         int size = mMediaList.size();
-        if (size == 0 || mCurrentIndex < 0 || mCurrentIndex >= size) {
-            if (mCurrentIndex < 0)
-                saveCurrentMedia();
-            Log.w(TAG, "Warning: invalid next index, aborted !");
-            stop();
-            return;
-        }
-
+        if (size > 1) {
+            mPrevious.push(mCurrentIndex);
+            mCurrentIndex = mNextIndex;
+            Log.d(TAG, "setting current to "+mCurrentIndex);
+            if (size == 0 || mCurrentIndex < 0 || mCurrentIndex >= size) {
+                if (mCurrentIndex < 0)
+                    saveCurrentMedia();
+                Log.w(TAG, "Warning: invalid next index, aborted !");
+                stop();
+                return;
+            }
+        } else
+            setPosition(0f);
         playIndex(mCurrentIndex, 0);
         onMediaChanged();
     }
 
     @MainThread
     public void previous() {
-        mCurrentIndex = mPrevIndex;
-        if (mPrevious.size() > 0)
-            mPrevious.pop();
-
         int size = mMediaList.size();
-        if (size == 0 || mPrevIndex < 0 || mCurrentIndex >= size) {
-            Log.w(TAG, "Warning: invalid previous index, aborted !");
-            stop();
-            return;
-        }
+        if (size > 1) {
+            mCurrentIndex = mPrevIndex;
+            if (mPrevious.size() > 0)
+                mPrevious.pop();
+            if (size == 0 || mPrevIndex < 0 || mCurrentIndex >= size) {
+                Log.w(TAG, "Warning: invalid previous index, aborted !");
+                stop();
+                return;
+            }
+        } else
+            setPosition(0f);
 
         playIndex(mCurrentIndex, 0);
         onMediaChanged();
@@ -1198,14 +1176,14 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         if (mShuffling)
             mPrevious.clear();
         mShuffling = !mShuffling;
-        saveCurrentMedia();
+        savePosition();
         determinePrevAndNextIndices();
     }
 
     @MainThread
-    public void setRepeatType(RepeatType t) {
-        mRepeating = t;
-        saveCurrentMedia();
+    public void setRepeatType(int repeatType) {
+        mRepeating = repeatType;
+        savePosition();
         determinePrevAndNextIndices();
     }
 
@@ -1215,8 +1193,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     }
 
     private void updateWidgetState() {
-        Intent i = new Intent(this, VLCAppWidgetProvider.class);
-        i.setAction(ACTION_WIDGET_UPDATE);
+        Intent i = new Intent(ACTION_WIDGET_UPDATE);
 
         if (hasCurrentMedia()) {
             final MediaWrapper media = getCurrentMedia();
@@ -1226,7 +1203,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                     : Util.getMediaArtist(this, media));
         }
         else {
-            i.putExtra("title", getString(R.string.widget_name));
+            i.putExtra("title", getString(R.string.widget_default_text));
             i.putExtra("artist", "");
         }
         i.putExtra("isplaying", mMediaPlayer.isPlaying());
@@ -1235,8 +1212,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     }
 
     private void updateWidgetCover() {
-        Intent i = new Intent(this, VLCAppWidgetProvider.class);
-        i.setAction(ACTION_WIDGET_UPDATE_COVER);
+        Intent i = new Intent(ACTION_WIDGET_UPDATE_COVER);
 
         Bitmap cover = hasCurrentMedia() ? AudioUtil.getCover(this, getCurrentMedia(), 64) : null;
         i.putExtra("cover", cover);
@@ -1254,8 +1230,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         updateWidgetState();
 
         mWidgetPositionTimestamp = timestamp;
-        Intent i = new Intent(this, VLCAppWidgetProvider.class);
-        i.setAction(ACTION_WIDGET_UPDATE_POSITION);
+        Intent i = new Intent(ACTION_WIDGET_UPDATE_POSITION);
         i.putExtra("position", pos);
         sendBroadcast(i);
     }
@@ -1289,7 +1264,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
             mediaPathList.add(Uri.decode(locations[i]));
 
         mShuffling = prefs.getBoolean("shuffling", false);
-        mRepeating = RepeatType.values()[prefs.getInt("repeating", RepeatType.None.ordinal())];
+        mRepeating = prefs.getInt("repeating", REPEAT_NONE);
         int position = prefs.getInt("position_in_list", Math.max(0, mediaPathList.indexOf(currentMedia)));
         long time = prefs.getLong("position_in_song", -1);
         // load playlist
@@ -1305,12 +1280,12 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     private synchronized void saveCurrentMedia() {
         SharedPreferences.Editor editor = PreferenceManager.getDefaultSharedPreferences(this).edit();
         editor.putString("current_media", mMediaList.getMRL(Math.max(mCurrentIndex, 0)));
-        editor.putBoolean("shuffling", mShuffling);
-        editor.putInt("repeating", mRepeating.ordinal());
         Util.commitPreferences(editor);
     }
 
     private synchronized void saveMediaList() {
+        if (getCurrentMedia() == null || getCurrentMedia().getType() == MediaWrapper.TYPE_VIDEO)
+            return;
         StringBuilder locations = new StringBuilder();
         for (int i = 0; i < mMediaList.size(); i++)
             locations.append(" ").append(Uri.encode(mMediaList.getMRL(i)));
@@ -1321,7 +1296,11 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     }
 
     private synchronized void savePosition(){
+        if (getCurrentMedia() == null || getCurrentMedia().getType() == MediaWrapper.TYPE_VIDEO)
+            return;
         SharedPreferences.Editor editor = PreferenceManager.getDefaultSharedPreferences(this).edit();
+        editor.putBoolean("shuffling", mShuffling);
+        editor.putInt("repeating", mRepeating);
         editor.putInt("position_in_list", mCurrentIndex);
         editor.putLong("position_in_song", mMediaPlayer.getTime());
         Util.commitPreferences(editor);
@@ -1369,7 +1348,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     }
 
     @MainThread
-    public RepeatType getRepeatType() {
+    public int getRepeatType() {
         return mRepeating;
     }
 
@@ -1530,6 +1509,9 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     public void load(List<MediaWrapper> mediaList, int position) {
         Log.v(TAG, "Loading position " + ((Integer) position).toString() + " in " + mediaList.toString());
 
+        if (hasCurrentMedia())
+            savePosition();
+
         mMediaList.removeEventListener(mListEventListener);
         mMediaList.clear();
         MediaWrapperList currentMediaList = mMediaList;
@@ -1884,7 +1866,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
 
         if (ml.getCount() > 0) {
             mMediaList.remove(mCurrentIndex);
-            for (int i = 0; i < ml.getCount(); ++i) {
+            for (int i = ml.getCount() - 1; i >= 0; --i) {
                 final Media child = ml.getMediaAt(i);
                 child.parse();
                 mMediaList.insert(mCurrentIndex, new MediaWrapper(child));
