@@ -135,9 +135,13 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
             Util.commitPreferences(editor);
         }
 
-        /* Load media items from database and storage */
         mMediaLibrary = MediaLibrary.getInstance();
-        mMediaLibrary.loadMediaItems();
+        if (mMediaLibrary.getMediaItems().isEmpty()) {
+            if (mSettings.getBoolean(PreferencesActivity.AUTO_RESCAN, true))
+                mMediaLibrary.scanMediaItems();
+            else
+                mMediaLibrary.loadMedaItems();
+        }
 
         /*** Start initializing the UI ***/
 
@@ -219,17 +223,12 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
     protected void onResume() {
         super.onResume();
 
-        /* FIXME: this is used to avoid having MainActivity twice in the backstack */
-        if (getIntent() != null && getIntent().hasExtra(PlaybackService.START_FROM_NOTIFICATION))
-            getIntent().removeExtra(PlaybackService.START_FROM_NOTIFICATION);
-
-
         /* Load media items from database and storage */
         if (mScanNeeded)
-            mMediaLibrary.loadMediaItems();
+            mMediaLibrary.scanMediaItems();
         if (mSlidingPane.getState() == mSlidingPane.STATE_CLOSED)
             mActionBar.hide();
-   }
+    }
 
     @Override
     protected void onResumeFragments() {
@@ -273,7 +272,6 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
     @Override
     protected void onPause() {
         super.onPause();
-
         /* Check for an ongoing scan that needs to be resumed during onResume */
         mScanNeeded = mMediaLibrary.isWorking();
         /* Stop scanning for files */
@@ -300,7 +298,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
 
     @Override
     public void onBackPressed() {
-            /* Close the menu first */
+        /* Close the menu first */
         if(mDrawerLayout.isDrawerOpen(mListView)) {
             if (mFocusedPrior != 0)
                 requestFocusOnSearch();
@@ -308,7 +306,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
             return;
         }
 
-        // Slide down the audio player if it is shown entirely.
+        /* Slide down the audio player if it is shown entirely. */
         if (slideDownAudioPlayer())
             return;
 
@@ -493,7 +491,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
                     if(current != null && current instanceof IRefreshable)
                         ((IRefreshable) current).refresh();
                     else
-                        mMediaLibrary.loadMediaItems(true);
+                        mMediaLibrary.scanMediaItems(true);
                 }
                 break;
             // Restore last playlist
@@ -536,9 +534,9 @@ public class MainActivity extends AudioPlayerContainerActivity implements OnItem
         super.onActivityResult(requestCode, resultCode, data);
         if (requestCode == ACTIVITY_RESULT_PREFERENCES) {
             if (resultCode == PreferencesActivity.RESULT_RESCAN)
-                mMediaLibrary.loadMediaItems(true);
+                mMediaLibrary.scanMediaItems(true);
             else if (resultCode == PreferencesActivity.RESULT_RESTART) {
-                Intent intent = getIntent();
+                Intent intent = new Intent(getIntent());
                 finish();
                 startActivity(intent);
             }
