@@ -73,6 +73,7 @@ public class PreferencesActivity extends PreferenceActivity implements OnSharedP
     public final static String VIDEO_SPEED = "VideoSpeed";
     public final static String VIDEO_BACKGROUND = "video_background";
     public final static String VIDEO_RESTORE = "video_restore";
+    public final static String AUTO_RESCAN = "auto_rescan";
     public final static int RESULT_RESCAN = RESULT_FIRST_USER + 1;
     public final static int RESULT_RESTART = RESULT_FIRST_USER + 2;
 
@@ -373,10 +374,9 @@ public class PreferencesActivity extends PreferenceActivity implements OnSharedP
     @Override
     public boolean onPreferenceTreeClick(PreferenceScreen preferenceScreen, Preference preference) {
         super.onPreferenceTreeClick(preferenceScreen, preference);
-        if (preference instanceof PreferenceScreen)
-            setUpNestedScreen((PreferenceScreen) preference);
         try {
             if (preference!=null && preference instanceof PreferenceScreen) {
+                setUpNestedScreen((PreferenceScreen) preference);
                 Dialog dialog = ((PreferenceScreen)preference).getDialog();
                 if (dialog!=null) {
                     Window window = dialog.getWindow();
