@@ -74,13 +74,10 @@ import java.util.Collections;
 import java.util.List;
 import java.util.Random;
 import java.util.concurrent.ConcurrentLinkedQueue;
-import java.util.concurrent.ExecutorService;
-import java.util.concurrent.Executors;
 
-public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeRefreshLayout.OnRefreshListener, MediaBrowser.EventListener, IBrowser {
+public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeRefreshLayout.OnRefreshListener, MediaBrowser.EventListener, IBrowser, ViewPager.OnPageChangeListener {
     public final static String TAG = "VLC/AudioBrowserFragment";
 
-    private MediaLibrary mMediaLibrary;
     private MediaBrowser mMediaBrowser;
     private MainActivity mMainActivity;
 
@@ -118,8 +115,6 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
     public void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
 
-        mMediaLibrary = MediaLibrary.getInstance();
-
         mSongsAdapter = new AudioBrowserListAdapter(getActivity(), AudioBrowserListAdapter.ITEM_WITH_COVER);
         mArtistsAdapter = new AudioBrowserListAdapter(getActivity(), AudioBrowserListAdapter.ITEM_WITH_COVER);
         mAlbumsAdapter = new AudioBrowserListAdapter(getActivity(), AudioBrowserListAdapter.ITEM_WITH_COVER);
@@ -164,23 +159,7 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
         mTabLayout = (TabLayout) v.findViewById(R.id.sliding_tabs);
         mTabLayout.setupWithViewPager(mViewPager);
 
-        mViewPager.addOnPageChangeListener(new TabLayout.TabLayoutOnPageChangeListener(mTabLayout));
-        mTabLayout.setOnTabSelectedListener(new TabLayout.OnTabSelectedListener() {
-            @Override
-            public void onTabSelected(TabLayout.Tab tab) {
-                mViewPager.setCurrentItem(tab.getPosition());
-                updateEmptyView(tab.getPosition());
-                setFabPlayShuffleAllVisibility();
-            }
-
-            @Override
-            public void onTabUnselected(TabLayout.Tab tab) {}
-
-            @Override
-            public void onTabReselected(TabLayout.Tab tab) {
-                mViewPager.setCurrentItem(tab.getPosition());
-            }
-        });
+        mViewPager.addOnPageChangeListener(this);
 
         songsList.setOnItemClickListener(songListener);
         artistList.setOnItemClickListener(artistListListener);
@@ -259,8 +238,10 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
         else if (mGenresAdapter.isEmpty() || mArtistsAdapter.isEmpty() ||
                 mAlbumsAdapter.isEmpty() || mSongsAdapter.isEmpty())
             updateLists();
-        else
+        else {
+            updateEmptyView(mViewPager.getCurrentItem());
             focusHelper(false, mLists.get(mViewPager.getCurrentItem()).getId());
+        }
         mMediaLibrary.addUpdateHandler(mHandler);
         mMediaLibrary.setBrowser(this);
         final ListView current = (ListView)mLists.get(mViewPager.getCurrentItem());
@@ -445,7 +426,7 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
                 return false;
             Message msg;
             if (mode == MODE_PLAYLIST) {
-                Snackbar.make(getView(), getString(R.string.playlist_deleted), Snackbar.LENGTH_LONG)
+                Snackbar.make(getView(), getString(R.string.playlist_deleted), DELETE_DURATION)
                 .setAction(android.R.string.cancel, mCancelDeletePlayListListener)
                 .show();
                 msg = mHandler.obtainMessage(DELETE_PLAYLIST, position, 0);
@@ -565,7 +546,7 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
     @Override
     public void onRefresh() {
         if (!MediaLibrary.getInstance().isWorking())
-            MediaLibrary.getInstance().loadMediaItems(true);
+            MediaLibrary.getInstance().scanMediaItems(true);
     }
 
     @Override
@@ -655,6 +636,26 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
         mMainActivity.sendTextInfo(info, progress, max);
     }
 
+    TabLayout.TabLayoutOnPageChangeListener tcl = new TabLayout.TabLayoutOnPageChangeListener(mTabLayout);
+
+    @Override
+    public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {
+        tcl.onPageScrolled(position, positionOffset, positionOffsetPixels);
+    }
+
+    @Override
+    public void onPageSelected(int position) {
+        mViewPager.setCurrentItem(position);
+        updateEmptyView(position);
+        setFabPlayShuffleAllVisibility();
+        tcl.onPageSelected(position);
+    }
+
+    @Override
+    public void onPageScrollStateChanged(int state) {
+        tcl.onPageScrollStateChanged(state);
+    }
+
     private static class AudioBrowserHandler extends WeakHandler<AudioBrowserFragment> {
         public AudioBrowserHandler(AudioBrowserFragment owner) {
             super(owner);
@@ -678,11 +679,11 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
                     MediaWrapper mw = fragment.mSongsAdapter.getItem(msg.arg1).mMediaList.get(0);
                     final String path = mw.getUri().getPath();
                     //Let's keep this toast while duration is not set correctly
-                    new Thread(new Runnable() {
+                    VLCApplication.runBackground(new Runnable() {
                         public void run() {
                             Util.recursiveDelete(VLCApplication.getAppContext(), new File(path));
                         }
-                    }).start();
+                    });
                     fragment.mMediaLibrary.getMediaItems().remove(mw);
                     refresh(fragment, path);
                     break;
@@ -713,7 +714,6 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
     }
 
     private void updateLists() {
-        clear();
         mAudioList = MediaLibrary.getInstance().getAudioItems();
         if (mAudioList.isEmpty()){
             updateEmptyView(mViewPager.getCurrentItem());
@@ -724,8 +724,7 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
             mTabLayout.setVisibility(View.VISIBLE);
             mHandler.sendEmptyMessageDelayed(MSG_LOADING, 300);
 
-            ExecutorService tpe = Executors.newSingleThreadExecutor();
-            ArrayList<Runnable> tasks = new ArrayList<Runnable>(Arrays.asList(updateArtists,
+            final ArrayList<Runnable> tasks = new ArrayList<Runnable>(Arrays.asList(updateArtists,
                     updateAlbums, updateSongs, updateGenres, updatePlaylists));
 
             //process the visible list first
@@ -737,8 +736,13 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
                         display();
                 }
             });
-            for (Runnable task : tasks)
-                tpe.submit(task);
+            VLCApplication.runBackground(new Runnable() {
+                @Override
+                public void run() {
+                    for (Runnable task : tasks)
+                        task.run();
+                }
+            });
         }
     }
 
