@@ -24,34 +24,40 @@ import android.app.Activity;
 import android.content.Context;
 import android.content.SharedPreferences;
 import android.database.DataSetObserver;
+import android.databinding.DataBindingUtil;
+import android.databinding.ViewDataBinding;
 import android.graphics.Bitmap;
+import android.graphics.drawable.BitmapDrawable;
 import android.preference.PreferenceManager;
+import android.support.v4.util.ArrayMap;
 import android.util.SparseArray;
 import android.view.LayoutInflater;
 import android.view.View;
-import android.view.View.OnClickListener;
 import android.view.ViewGroup;
 import android.widget.BaseAdapter;
 import android.widget.ImageView;
-import android.widget.RelativeLayout;
+import android.widget.LinearLayout;
 import android.widget.SectionIndexer;
 import android.widget.TextView;
 
+import org.videolan.vlc.BR;
 import org.videolan.vlc.MediaWrapper;
 import org.videolan.vlc.R;
-import org.videolan.vlc.util.BitmapCache;
+import org.videolan.vlc.VLCApplication;
+import org.videolan.vlc.gui.AsyncImageLoader;
+import org.videolan.vlc.interfaces.IAudioClickHandler;
+import org.videolan.vlc.util.BitmapUtil;
 import org.videolan.vlc.util.Util;
 
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
-import java.util.HashMap;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 
-public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndexer {
+public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndexer, IAudioClickHandler {
     public final static String TAG = "VLC/AudioBrowserListAdapter";
 
     public final static int TYPE_ARTISTS = 0;
@@ -101,8 +107,8 @@ public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndex
     }
 
     public AudioBrowserListAdapter(Activity context, int itemType) {
-        mMediaItemMap = new HashMap<String, ListItem>();
-        mSeparatorItemMap = new HashMap<String, ListItem>();
+        mMediaItemMap = new ArrayMap<String, ListItem>();
+        mSeparatorItemMap = new ArrayMap<String, ListItem>();
         mItems = new ArrayList<ListItem>();
         mSections = new SparseArray<String>();
         mContext = context;
@@ -113,12 +119,17 @@ public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndex
         mAlignMode = Integer.valueOf(preferences.getString("audio_title_alignment", "0"));
     }
 
-    public void addAll(List<ListItem> items) {
-        for (ListItem item : items) {
-            mMediaItemMap.put(item.mTitle, item);
-            mItems.add(item);
-        }
-        Collections.sort(mItems, mItemsComparator);
+    public void addAll(final List<ListItem> items) {
+        mContext.runOnUiThread(new Runnable() {
+            @Override
+            public void run() {
+                for (ListItem item : items) {
+                    mMediaItemMap.put(item.mTitle, item);
+                    mItems.add(item);
+                }
+                Collections.sort(mItems, mItemsComparator);
+            }
+        });
     }
 
     public void add(String title, String subTitle, MediaWrapper media) {
@@ -148,6 +159,7 @@ public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndex
         mContext.runOnUiThread(new Runnable() {
             @Override
             public void run() {
+                clear();
                 String title, subTitle, key;
                 for (MediaWrapper media : list) {
                     switch (type){
@@ -300,7 +312,7 @@ public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndex
 
     public View getViewMedia(int position, View convertView, ViewGroup parent) {
         View v = convertView;
-        ViewHolder holder;
+        ViewHolder holder = null;
 
         /* convertView may be a recycled view but we must recreate it
          * if it does not correspond to a media view. */
@@ -313,70 +325,46 @@ public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndex
 
         if (b_createView) {
             LayoutInflater inflater = (LayoutInflater) mContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
-            v = inflater.inflate(R.layout.audio_browser_item, parent, false);
+
             holder = new ViewHolder();
-            holder.layout = v.findViewById(R.id.layout_item);
-            holder.title = (TextView) v.findViewById(R.id.title);
-            Util.setAlignModeByPref(mAlignMode, holder.title);
-            holder.cover = (ImageView) v.findViewById(R.id.cover);
-            holder.subtitle = (TextView) v.findViewById(R.id.subtitle);
-            holder.footer = v.findViewById(R.id.footer);
-            holder.more = (ImageView) v.findViewById(R.id.item_more);
+            holder.binding = DataBindingUtil.inflate(inflater, R.layout.audio_browser_item, parent, false);
+            v = holder.binding.getRoot();
+            Util.setAlignModeByPref(mAlignMode, (TextView) v.findViewById(R.id.title));
             holder.viewType = VIEW_MEDIA;
             v.setTag(holder);
-        } else
-            holder = (ViewHolder) v.getTag();
-
-        ListItem item = getItem(position);
-        holder.title.setText(item.mTitle);
-
-        RelativeLayout.LayoutParams paramsCover;
-        if (mItemType == ITEM_WITH_COVER) {
-            Bitmap cover = null;
-            LinkedList<String> testedAlbums = new LinkedList<String>();
-            for (MediaWrapper media : mItems.get(position).mMediaList) {
-                if (media.getAlbum() != null && testedAlbums.contains(media.getAlbum()))
-                    continue;
-                cover = AudioUtil.getCover(v.getContext(), media, 64);
-                if (cover != null)
-                    break;
-                else if (media.getAlbum() != null)
-                    testedAlbums.add(media.getAlbum());
-            }
-            if (cover == null)
-                cover = BitmapCache.getFromResource(v, R.drawable.icon);
-            holder.cover.setImageBitmap(cover);
-            int size = (int) mContext.getResources().getDimension(R.dimen.audio_browser_item_size);
-            paramsCover = new RelativeLayout.LayoutParams(size, size);
         }
-        else
-            paramsCover = new RelativeLayout.LayoutParams(0, RelativeLayout.LayoutParams.WRAP_CONTENT);
-        holder.cover.setLayoutParams(paramsCover);
 
-        holder.subtitle.setVisibility(item.mSubTitle == null ? TextView.GONE : TextView.VISIBLE);
-        holder.subtitle.setText(item.mSubTitle);
+        holder.position = position;
+        ListItem item = getItem(position);
+        holder.binding.setVariable(BR.item, item);
 
-        // Remove the footer if the item is just above a separator.
-        holder.footer.setVisibility(isMediaItemAboveASeparator(position) ? View.GONE : View.VISIBLE);
+        final ArrayList<MediaWrapper> mediaList = mItems.get(position).mMediaList;
+        boolean asyncLoad = true;
 
-        final int pos = position;
-        if (mContextPopupMenuListener != null)
-            holder.more.setOnClickListener(new OnClickListener() {
-                @Override
-                public void onClick(View v) {
-                    if (mContextPopupMenuListener != null)
-                        mContextPopupMenuListener.onPopupMenu(v, pos);
-                }
-            });
-        else
-            holder.more.setVisibility(View.GONE);
+        if (mItemType == ITEM_WITH_COVER) {
+            Bitmap bitmap = AudioUtil.getCoverFromMemCache(mContext, mediaList, 64);
+            if (bitmap != null) {
+                asyncLoad = false;
+                holder.binding.setVariable(BR.cover, new BitmapDrawable(VLCApplication.getAppResources(), bitmap));
+            } else {
+                holder.binding.setVariable(BR.cover, AudioUtil.DEFAULT_COVER);
+            }
+        } else
+            holder.binding.setVariable(BR.cover, AudioUtil.DEFAULT_COVER);
+
+        holder.binding.setVariable(BR.footer, !isMediaItemAboveASeparator(position));
+        holder.binding.setVariable(BR.clickable, mContextPopupMenuListener != null);
+        holder.binding.setVariable(BR.handler, this);
+        holder.binding.executePendingBindings();
+        if (asyncLoad)
+            AsyncImageLoader.LoadImage(new AudioCoverFetcher(holder.binding, mContext, mediaList), null);
 
         return v;
     }
 
     public View getViewSeparator(int position, View convertView, ViewGroup parent) {
         View v = convertView;
-        ViewHolder holder;
+        ViewHolder holder = null;
 
         /* convertView may be a recycled view but we must recreate it
          * if it does not correspond to a separator view. */
@@ -389,29 +377,24 @@ public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndex
 
         if (b_createView) {
             LayoutInflater inflater = (LayoutInflater) mContext.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
-            v = inflater.inflate(R.layout.audio_browser_separator, parent, false);
             holder = new ViewHolder();
-            holder.layout = v.findViewById(R.id.layout_item);
-            holder.title = (TextView) v.findViewById(R.id.title);
+            holder.binding = DataBindingUtil.inflate(inflater, R.layout.audio_browser_separator, parent, false);
+            v = holder.binding.getRoot();
             holder.viewType = VIEW_SEPARATOR;
             v.setTag(holder);
-        } else
-            holder = (ViewHolder) v.getTag();
+        }
 
         ListItem item = getItem(position);
-        holder.title.setText(item.mTitle);
+        holder.binding.setVariable(BR.item, item);
+        holder.binding.executePendingBindings();
 
         return v;
     }
 
     static class ViewHolder {
-        View layout;
-        ImageView cover;
-        TextView title;
-        TextView subtitle;
-        View footer;
-        ImageView more;
+        int position;
         int viewType;
+        ViewDataBinding binding;
     }
 
     @Override
@@ -586,4 +569,34 @@ public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndex
             return String.CASE_INSENSITIVE_ORDER.compare(lhs.mTitle, rhs.mTitle);
         }
     };
+
+    @Override
+    public void onMoreClick(View v) {
+        if (mContextPopupMenuListener != null)
+            mContextPopupMenuListener.onPopupMenu(v, ((ViewHolder) ((LinearLayout)v.getParent().getParent()).getTag()).position);
+    }
+
+    private static class AudioCoverFetcher extends AsyncImageLoader.CoverFetcher {
+        final ArrayList<MediaWrapper> list;
+        final Context context;
+
+        AudioCoverFetcher(ViewDataBinding binding, Context context, ArrayList<MediaWrapper> list) {
+            super(binding);
+            this.context = context;
+            this.list = list;
+        }
+
+        @Override
+        public Bitmap getImage() {
+            return AudioUtil.getCover(context, list, 64);
+        }
+
+        @Override
+        public void updateBindImage(Bitmap bitmap, View target) {
+            if (bitmap != null && (bitmap.getWidth() != 1 && bitmap.getHeight() != 1)) {
+                binding.setVariable(BR.scaleType, ImageView.ScaleType.FIT_CENTER);
+                binding.setVariable(BR.cover, new BitmapDrawable(VLCApplication.getAppResources(), bitmap));
+            }
+        }
+    }
 }
