@@ -146,11 +146,11 @@ public class AudioPlaylistAdapter extends ArrayAdapter<MediaWrapper> {
         return location;
     }
 
-    public List<String> getLocations() {
-        List<String> locations = new ArrayList<String>();
+    public List<MediaWrapper> getMedias() {
+        List<MediaWrapper> medias = new ArrayList<MediaWrapper>();
         for (int i = 0 ; i < getCount() ; ++i)
-            locations.add(getItem(i).getLocation());
-        return locations;
+            medias.add(getItem(i));
+        return medias;
     }
 
     static class ViewHolder {
