@@ -27,9 +27,11 @@ import android.database.Cursor;
 import android.graphics.Bitmap;
 import android.graphics.Bitmap.CompressFormat;
 import android.graphics.BitmapFactory;
+import android.graphics.drawable.BitmapDrawable;
 import android.media.RingtoneManager;
 import android.net.Uri;
 import android.provider.MediaStore;
+import android.support.annotation.RequiresPermission;
 import android.util.Log;
 import android.widget.Toast;
 
@@ -40,6 +42,7 @@ import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.util.AndroidDevices;
 import org.videolan.vlc.util.BitmapCache;
+import org.videolan.vlc.util.BitmapUtil;
 import org.videolan.vlc.util.MurmurHash;
 import org.videolan.vlc.util.Util;
 
@@ -53,7 +56,9 @@ import java.io.UnsupportedEncodingException;
 import java.math.BigInteger;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
+import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.LinkedList;
 
 public class AudioUtil {
     public final static String TAG = "VLC/AudioUtil";
@@ -75,6 +80,9 @@ public class AudioUtil {
      */
     public static String PLAYLIST_DIR = null;
 
+    public static final BitmapDrawable DEFAULT_COVER = new BitmapDrawable(VLCApplication.getAppResources(), BitmapCache.getFromResource(VLCApplication.getAppResources(), R.drawable.icon));
+
+    @RequiresPermission(android.Manifest.permission.WRITE_SETTINGS)
     public static void setRingtone(MediaWrapper song, Context context){
         File newringtone = AndroidUtil.UriToFile(song.getUri());
         if(newringtone == null || !newringtone.exists()) {
@@ -101,7 +109,7 @@ public class AudioUtil {
                     context.getApplicationContext(),
                     RingtoneManager.TYPE_RINGTONE,
                     newUri
-                    );
+            );
         } catch(Exception e) {
             Toast.makeText(context.getApplicationContext(),
                     context.getString(R.string.ringtone_error),
@@ -165,11 +173,11 @@ public class AudioUtil {
             return null;
         ContentResolver contentResolver = context.getContentResolver();
         Uri uri = android.provider.MediaStore.Audio.Albums.EXTERNAL_CONTENT_URI;
-        Cursor cursor = contentResolver.query(uri, new String[] {
-                       MediaStore.Audio.Albums.ALBUM,
-                       MediaStore.Audio.Albums.ALBUM_ART },
-                       MediaStore.Audio.Albums.ALBUM + " LIKE ?",
-                       new String[] { album }, null);
+        Cursor cursor = contentResolver.query(uri, new String[]{
+                        MediaStore.Audio.Albums.ALBUM,
+                        MediaStore.Audio.Albums.ALBUM_ART},
+                MediaStore.Audio.Albums.ALBUM + " LIKE ?",
+                new String[]{album}, null);
         if (cursor == null) {
             // do nothing
         } else if (!cursor.moveToFirst()) {
@@ -195,8 +203,8 @@ public class AudioUtil {
 
             /* Parse decoded attachment */
             if( mArtist.length() == 0 || mAlbum.length() == 0 ||
-                mArtist.equals(VLCApplication.getAppContext().getString(R.string.unknown_artist)) ||
-                mAlbum.equals(VLCApplication.getAppContext().getString(R.string.unknown_album)) )
+                    mArtist.equals(VLCApplication.getAppContext().getString(R.string.unknown_artist)) ||
+                    mAlbum.equals(VLCApplication.getAppContext().getString(R.string.unknown_album)) )
             {
                 /* If artist or album are missing, it was cached by title MD5 hash */
                 MessageDigest md = MessageDigest.getInstance("MD5");
@@ -265,6 +273,19 @@ public class AudioUtil {
         return null;
     }
 
+    private static String getCoverCachePath(Context context, MediaWrapper media, int width) {
+        final int hash = MurmurHash.hash32(Util.getMediaArtist(context, media) + Util.getMediaAlbum(context, media));
+        return COVER_DIR + (hash >= 0 ? "" + hash : "m" + (-hash)) + "_" + width;
+    }
+
+    public static Bitmap getCoverFromMemCache(Context context, MediaWrapper media, int width) {
+        if (media != null && media.getArtist() != null && media.getAlbum() != null) {
+            final BitmapCache cache = BitmapCache.getInstance();
+            return cache.getBitmapFromMemCache(getCoverCachePath(context, media, width));
+        } else
+            return null;
+    }
+
     @SuppressLint("NewApi")
     public synchronized static Bitmap getCover(Context context, MediaWrapper media, int width) {
         BitmapCache cache = BitmapCache.getInstance();
@@ -285,8 +306,7 @@ public class AudioUtil {
         try {
             // try to load from cache
             if (media.getArtist() != null && media.getAlbum() != null) {
-                int hash = MurmurHash.hash32(Util.getMediaArtist(context, media) + Util.getMediaAlbum(context, media));
-                cachePath = COVER_DIR + (hash >= 0 ? "" + hash : "m" + (-hash)) + "_" + width;
+                cachePath = getCoverCachePath(context, media, width);
 
                 // try to get the cover from the LRUCache first
                 cover = cache.getBitmapFromMemCache(cachePath);
@@ -295,7 +315,7 @@ public class AudioUtil {
 
                 // try to get the cover from the storage cache
                 cacheFile = new File(cachePath);
-                if (cacheFile != null && cacheFile.exists()) {
+                if (cacheFile.exists()) {
                     if (cacheFile.length() > 0)
                         coverPath = cachePath;
                     else
@@ -307,14 +327,14 @@ public class AudioUtil {
             if (coverPath == null || !cacheFile.exists())
                 coverPath = getCoverFromVlc(context, media);
 
-            // no found yet, looking in folder
-            if (coverPath == null || !(new File(coverPath)).exists())
-                coverPath = getCoverFromFolder(media);
-
             // try to get the cover from android MediaStore
             if (coverPath == null || !(new File(coverPath)).exists())
                 coverPath = getCoverFromMediaStore(context, media);
 
+            // no found yet, looking in folder
+            if (coverPath == null || !(new File(coverPath)).exists())
+                coverPath = getCoverFromFolder(media);
+
             // read (and scale?) the bitmap
             cover = readCoverBitmap(coverPath, width);
 
@@ -353,6 +373,9 @@ public class AudioUtil {
 
         /* Get the resolution of the bitmap without allocating the memory */
         options.inJustDecodeBounds = true;
+        if (AndroidUtil.isHoneycombOrLater())
+            options.inMutable = true;
+        BitmapUtil.setInBitmap(options);
         BitmapFactory.decodeFile(path, options);
 
         if (options.outWidth > 0 && options.outHeight > 0) {
@@ -364,9 +387,37 @@ public class AudioUtil {
                 options.inSampleSize = options.inSampleSize * 2;
 
             // Decode the file (with memory allocation this time)
+            BitmapUtil.setInBitmap(options);
             cover = BitmapFactory.decodeFile(path, options);
         }
 
         return cover;
     }
+
+    public static Bitmap getCover(Context context, ArrayList<MediaWrapper> list, int width, boolean fromMemCache) {
+        Bitmap cover = null;
+        LinkedList<String> testedAlbums = new LinkedList<String>();
+        for (MediaWrapper media : list) {
+            /* No list cover is artist or album are null */
+            if (media.getAlbum() == null || media.getArtist() == null)
+                continue;
+            if (testedAlbums.contains(media.getAlbum()))
+                continue;
+
+            cover = fromMemCache ? AudioUtil.getCoverFromMemCache(context, media, width) : AudioUtil.getCover(context, media, width);
+            if (cover != null)
+                break;
+            else if (media.getAlbum() != null)
+                testedAlbums.add(media.getAlbum());
+        }
+        return cover;
+    }
+
+    public static Bitmap getCoverFromMemCache(Context context, ArrayList<MediaWrapper> list, int width) {
+        return getCover(context, list, width, true);
+    }
+
+    public static Bitmap getCover(Context context, ArrayList<MediaWrapper> list, int width) {
+        return getCover(context, list, width, false);
+    }
 }
