@@ -19,10 +19,8 @@
  *****************************************************************************/
 package org.videolan.vlc.gui.audio;
 
-import android.content.SharedPreferences;
+import android.content.Context;
 import android.os.Bundle;
-import android.preference.PreferenceManager;
-import android.support.v4.app.Fragment;
 import android.support.v7.app.AppCompatActivity;
 import android.support.v7.widget.SwitchCompat;
 import android.view.LayoutInflater;
@@ -39,24 +37,23 @@ import android.widget.SeekBar;
 import android.widget.SeekBar.OnSeekBarChangeListener;
 import android.widget.Spinner;
 
-import org.videolan.libvlc.LibVLC;
+import org.videolan.libvlc.MediaPlayer;
+import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
-import org.videolan.vlc.VLCApplication;
+import org.videolan.vlc.gui.PlaybackServiceFragment;
 import org.videolan.vlc.interfaces.OnEqualizerBarChangeListener;
-import org.videolan.vlc.util.Preferences;
-import org.videolan.vlc.util.Util;
-import org.videolan.vlc.util.VLCInstance;
+import org.videolan.vlc.util.VLCOptions;
 import org.videolan.vlc.widget.EqualizerBar;
 
-public class EqualizerFragment extends Fragment {
+public class EqualizerFragment extends PlaybackServiceFragment {
 
     public final static String TAG = "VLC/EqualizerFragment";
     private SwitchCompat button;
     private Spinner equalizer_presets;
     private SeekBar preamp;
     private LinearLayout bands_layout;
-    LibVLC libVlc = null;
-    float[] equalizer = null;
+    private MediaPlayer.Equalizer mEqualizer = null;
+    private static final int BAND_COUNT = MediaPlayer.Equalizer.getBandCount();
 
     /* All subclasses of Fragment must include a public empty constructor. */
     public EqualizerFragment() {
@@ -69,6 +66,7 @@ public class EqualizerFragment extends Fragment {
         super.onCreateView(inflater, container, savedInstanceState);
         View v = inflater.inflate(R.layout.equalizer, container, false);
         saveViewChildren(v);
+        fillViews();
 
         return v;
     }
@@ -80,33 +78,32 @@ public class EqualizerFragment extends Fragment {
         bands_layout = (LinearLayout) v.findViewById(R.id.equalizer_bands);
     }
 
+
     @Override
-    public void onResume() {
-        super.onResume();
-        fillViews();
+    public void onConnected(PlaybackService service) {
+        super.onConnected(service);
     }
 
     private void fillViews() {
-        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext());
-        float[] bands = null;
-        String[] presets = null;
-        libVlc = VLCInstance.get();
+        final Context context = getActivity();
 
-        bands = libVlc.getBands();
-        presets = libVlc.getPresets();
-        if (equalizer == null)
-            equalizer = Preferences.getFloatArray(preferences, "equalizer_values");
-        if (equalizer == null)
-            equalizer = new float[bands.length + 1];
+        if (context == null)
+            return;
+
+        final String[] presets = getEqualizerPresets();
+
+        mEqualizer = VLCOptions.getEqualizer(context);
+        final boolean isEnabled = mEqualizer != null;
+        if (mEqualizer == null)
+            mEqualizer = MediaPlayer.Equalizer.create();
 
         // on/off
-        button.setChecked(libVlc.getEqualizer() != null);
+        button.setChecked(isEnabled);
         button.setOnCheckedChangeListener(new OnCheckedChangeListener() {
             @Override
             public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
-                if (libVlc == null)
-                    return;
-                libVlc.setEqualizer(isChecked ? equalizer : null);
+                if (mService != null)
+                    mService.setEqualizer(isChecked ? mEqualizer : null);
             }
         });
 
@@ -114,7 +111,7 @@ public class EqualizerFragment extends Fragment {
         equalizer_presets.setAdapter(new ArrayAdapter<String>(getActivity(), android.R.layout.simple_spinner_dropdown_item, presets));
 
         // Set the default selection asynchronously to prevent a layout initialization bug.
-        final int equalizer_preset_pref = preferences.getInt("equalizer_preset", 0);
+        final int equalizer_preset_pref = VLCOptions.getEqualizerPreset(context);
         equalizer_presets.post(new Runnable() {
             @Override
             public void run() {
@@ -125,16 +122,16 @@ public class EqualizerFragment extends Fragment {
 
         // preamp
         preamp.setMax(40);
-        preamp.setProgress((int) equalizer[0] + 20);
+        preamp.setProgress((int) mEqualizer.getPreAmp() + 20);
         preamp.setOnSeekBarChangeListener(mPreampListener);
 
         // bands
-        for (int i = 0; i < bands.length; i++) {
-            float band = bands[i];
+        for (int i = 0; i < BAND_COUNT; i++) {
+            float band = MediaPlayer.Equalizer.getBandFrequency(i);
 
             EqualizerBar bar = new EqualizerBar(getActivity(), band);
-            bar.setValue(equalizer[i + 1]);
-            bar.setListener(new BandListener(i + 1));
+            bar.setValue(mEqualizer.getAmp(i));
+            bar.setListener(new BandListener(i));
 
             bands_layout.addView(bar);
             LinearLayout.LayoutParams params =
@@ -152,27 +149,24 @@ public class EqualizerFragment extends Fragment {
         preamp.setOnSeekBarChangeListener(null);
         bands_layout.removeAllViews();
 
-        SharedPreferences.Editor editor = PreferenceManager.getDefaultSharedPreferences(VLCApplication.getAppContext()).edit();
-        editor.putBoolean("equalizer_enabled", button.isChecked());
-        Preferences.putFloatArray(editor, "equalizer_values", equalizer);
-        editor.putInt("equalizer_preset", equalizer_presets.getSelectedItemPosition());
-        Util.commitPreferences(editor);
+        if (button.isChecked())
+            VLCOptions.setEqualizer(getActivity(), mEqualizer, equalizer_presets.getSelectedItemPosition());
+        else
+            VLCOptions.setEqualizer(getActivity(), null, 0);
     }
 
     private final OnItemSelectedListener mPresetListener = new OnItemSelectedListener() {
         @Override
         public void onItemSelected(AdapterView<?> parent, View view, int pos, long id) {
-            if (libVlc == null)
-                return;
-            float[] preset = libVlc.getPreset(pos);
-            if (preset == null)
+            if (mService == null)
                 return;
 
-            equalizer = preset;
-            preamp.setProgress((int) equalizer[0] + 20);
-            for (int i = 0; i < equalizer.length - 1; ++i) {
+            mEqualizer = MediaPlayer.Equalizer.createFromPreset(pos);
+
+            preamp.setProgress((int) mEqualizer.getPreAmp() + 20);
+            for (int i = 0; i < BAND_COUNT; ++i) {
                 EqualizerBar bar = (EqualizerBar) bands_layout.getChildAt(i);
-                bar.setValue(equalizer[i + 1]);
+                bar.setValue(mEqualizer.getAmp(i));
             }
         }
 
@@ -192,12 +186,12 @@ public class EqualizerFragment extends Fragment {
 
         @Override
         public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
-            if (!fromUser)
+            if (!fromUser || mService == null)
                 return;
 
-            equalizer[0] = progress - 20;
-            if (libVlc != null && button.isChecked())
-                libVlc.setEqualizer(equalizer);
+            mEqualizer.setPreAmp(progress - 20);
+            if (button.isChecked())
+                mService.setEqualizer(mEqualizer);
         }
     };
 
@@ -210,9 +204,20 @@ public class EqualizerFragment extends Fragment {
 
         @Override
         public void onProgressChanged(float value) {
-            equalizer[index] = value;
-            if (libVlc != null && button.isChecked())
-                libVlc.setEqualizer(equalizer);
+            mEqualizer.setAmp(index, value);
+            if (button.isChecked() && mService != null)
+                mService.setEqualizer(mEqualizer);
         }
-    };
+    }
+
+    private static String[] getEqualizerPresets() {
+        final int count = MediaPlayer.Equalizer.getPresetCount();
+        if (count <= 0)
+            return null;
+        final String [] presets = new String[count];
+        for (int i = 0; i < count; ++i) {
+            presets[i] = MediaPlayer.Equalizer.getPresetName(i);
+        }
+        return presets;
+    }
 }
\ No newline at end of file
