@@ -29,10 +29,12 @@ import android.view.LayoutInflater;
 import android.view.View;
 import android.view.ViewGroup;
 import android.widget.CheckBox;
+import android.widget.LinearLayout;
 
 import org.videolan.libvlc.Media;
 import org.videolan.vlc.MediaWrapper;
 import org.videolan.vlc.R;
+import org.videolan.vlc.VLCApplication;
 
 public class StorageBrowserAdapter extends BaseBrowserAdapter {
 
@@ -59,24 +61,14 @@ public class StorageBrowserAdapter extends BaseBrowserAdapter {
         final Storage storage = (Storage) getItem(position);
         String storagePath = storage.getUri().getPath();
         boolean hasContextMenu = mCustomDirsLocation.contains(storagePath);
-        vh.title.setText(storage.getName());
-        vh.icon.setVisibility(View.GONE);
-        vh.checkBox.setVisibility(View.VISIBLE);
-        vh.more.setVisibility(hasContextMenu ? View.VISIBLE : View.GONE);
-        vh.text.setVisibility(View.GONE);
-
-        vh.itemView.setOnClickListener(new View.OnClickListener() {
-            @Override
-            public void onClick(View v) {
-                MediaWrapper mw = new MediaWrapper(((Storage) getItem(vh.getAdapterPosition())).getUri());
-                mw.setType(MediaWrapper.TYPE_DIR);
-                ((StorageBrowserFragment) fragment).browse(mw, holder.getAdapterPosition(), vh.checkBox.isChecked());
-            }
-        });
-        vh.checkBox.setChecked(((StorageBrowserFragment) fragment).mScannedDirectory ||
+        vh.binding.setHandler(mClickHandler);
+        vh.binding.setStorage(storage);
+        vh.binding.setHasContextMenu(hasContextMenu);
+        vh.binding.setType(TYPE_STORAGE);
+        vh.binding.setChecked(((StorageBrowserFragment) fragment).mScannedDirectory ||
                 (isRoot && (mMediaDirsLocation == null || mMediaDirsLocation.isEmpty())) ||
                 mMediaDirsLocation.contains(storagePath));
-        vh.checkBox.setEnabled(!((StorageBrowserFragment) fragment).mScannedDirectory);
+        vh.binding.setChechEnabled(!((StorageBrowserFragment) fragment).mScannedDirectory);
         vh.checkBox.setOnClickListener(new View.OnClickListener() {
             @Override
             public void onClick(View v) {
@@ -88,21 +80,8 @@ public class StorageBrowserAdapter extends BaseBrowserAdapter {
                     removeDir(path);
             }
         });
-        if (hasContextMenu) {
-            vh.more.setOnClickListener(new View.OnClickListener() {
-                @Override
-                public void onClick(View v) {
-                    fragment.onPopupMenu(vh.more, holder.getAdapterPosition());
-                }
-            });
-            vh.itemView.setOnLongClickListener(new View.OnLongClickListener() {
-                @Override
-                public boolean onLongClick(View v) {
-                    fragment.mRecyclerView.openContextMenu(holder.getAdapterPosition());
-                    return true;
-                }
-            });
-        }
+        if (hasContextMenu)
+            vh.itemView.setOnLongClickListener(this);
     }
 
     public void addItem(Media media, boolean notify, boolean top){
@@ -111,14 +90,14 @@ public class StorageBrowserAdapter extends BaseBrowserAdapter {
     }
 
     private void removeDir(final String path) {
-        new Thread(new Runnable() {
+        VLCApplication.runBackground(new Runnable() {
             @Override
             public void run() {
                 //if media dir list was empty, we add all others
                 if (mMediaDirsLocation.isEmpty()) {
                     Storage storage;
                     String pathString;
-                    for (Object item : mMediaList){
+                    for (Object item : mMediaList) {
                         storage = (Storage) item;
                         if (!TextUtils.equals(path, storage.getUri().getPath())) {
                             pathString = storage.getUri().getPath();
@@ -132,11 +111,11 @@ public class StorageBrowserAdapter extends BaseBrowserAdapter {
                 if (isRoot && mMediaDirsLocation.isEmpty() && getItemCount() > 1)
                     refreshFragment();
             }
-        }).start();
+        });
     }
 
     private void addDir(final String path) {
-        new Thread(new Runnable() {
+        VLCApplication.runBackground(new Runnable() {
             @Override
             public void run() {
                 mDbManager.addDir(path);
@@ -153,7 +132,7 @@ public class StorageBrowserAdapter extends BaseBrowserAdapter {
                 }
                 updateMediaDirs();
             }
-        }).start();
+        });
     }
 
     void refreshFragment(){
@@ -165,4 +144,21 @@ public class StorageBrowserAdapter extends BaseBrowserAdapter {
             }
         });
     }
+
+    protected void openMediaFromView(View v) {
+        MediaViewHolder holder = (MediaViewHolder) v.getTag(R.id.layout_item);
+        MediaWrapper mw = new MediaWrapper(((Storage) getItem(holder.getAdapterPosition())).getUri());
+        mw.setType(MediaWrapper.TYPE_DIR);
+        fragment.browse(mw, holder.getAdapterPosition(), holder.checkBox.isChecked());
+    }
+
+    protected void checkBoxAction(View v){
+        MediaViewHolder holder = (MediaViewHolder) ((LinearLayout)v.getParent()).getTag(R.id.layout_item);
+            boolean isChecked = ((CheckBox) v).isChecked();
+            String path = ((Storage) getItem(holder.getAdapterPosition())).getUri().getPath();
+            if (isChecked)
+                addDir(path);
+            else
+                removeDir(path);
+    }
 }
