@@ -35,6 +35,7 @@ import android.os.Message;
 import android.preference.PreferenceManager;
 import android.support.design.widget.Snackbar;
 import android.support.v4.app.FragmentActivity;
+import android.support.v4.util.ArrayMap;
 import android.util.DisplayMetrics;
 import android.util.Log;
 import android.view.ContextMenu;
@@ -75,10 +76,7 @@ import org.videolan.vlc.util.WeakHandler;
 import org.videolan.vlc.widget.SwipeRefreshLayout;
 
 import java.io.File;
-import java.util.HashMap;
 import java.util.List;
-import java.util.concurrent.BrokenBarrierException;
-import java.util.concurrent.CyclicBarrier;
 
 public class VideoGridFragment extends MediaBrowserFragment implements ISortable, IVideoBrowser, SwipeRefreshLayout.OnRefreshListener, AdapterView.OnItemClickListener {
 
@@ -89,25 +87,13 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     private static final int DELETE_MEDIA = 0;
     private static final int DELETE_DURATION = 3000;
 
-    protected static final String ACTION_SCAN_START = "org.videolan.vlc.gui.ScanStart";
-    protected static final String ACTION_SCAN_STOP = "org.videolan.vlc.gui.ScanStop";
-    protected static final int UPDATE_ITEM = 0;
-
-    /* Constants used to switch from Grid to List and vice versa */
-    //FIXME If you know a way to do this in pure XML please do it!
-    private static final int GRID_STRETCH_MODE = GridView.STRETCH_COLUMN_WIDTH;
-    private static final int LIST_STRETCH_MODE = GridView.STRETCH_COLUMN_WIDTH;
-
     protected LinearLayout mLayoutFlipperLoading;
     protected GridView mGridView;
     protected TextView mTextViewNomedia;
     protected View mViewNomedia;
-    protected MediaWrapper mItemToUpdate;
     protected String mGroup;
-    protected final CyclicBarrier mBarrier = new CyclicBarrier(2);
 
     private VideoListAdapter mVideoAdapter;
-    private MediaLibrary mMediaLibrary;
     private Thumbnailer mThumbnailer;
     private VideoGridAnimator mAnimator;
 
@@ -124,7 +110,6 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         super.onCreate(savedInstanceState);
 
         mVideoAdapter = new VideoListAdapter(this);
-        mMediaLibrary = MediaLibrary.getInstance();
 
         if (savedInstanceState != null)
             setGroup(savedInstanceState.getString(KEY_GROUP));
@@ -208,7 +193,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
             focusHelper(false);
         }
         //Get & set times
-        HashMap<String, Long> times = MediaDatabase.getInstance().getVideoTimes();
+        ArrayMap<String, Long> times = MediaDatabase.getInstance().getVideoTimes();
         mVideoAdapter.setTimes(times);
         mGridView.setSelection(mGVFirstVisiblePos);
         updateViewMode();
@@ -237,7 +222,6 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         super.onDestroy();
         if (mThumbnailer != null)
             mThumbnailer.clearJobs();
-        mBarrier.reset();
         mVideoAdapter.clear();
     }
 
@@ -245,7 +229,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         if (mGroup == null)
             return getString(R.string.video);
         else
-            return mGroup;
+            return mGroup + "\u2026";
     }
 
     private void updateViewMode() {
@@ -266,14 +250,12 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         if (!listMode) {
             sidePadding = (int) ((float)outMetrics.widthPixels / 100f * (float)Math.pow(outMetrics.density, 3) / 2f);
             mGridView.setNumColumns(GridView.AUTO_FIT);
-            mGridView.setStretchMode(GRID_STRETCH_MODE);
             mGridView.setColumnWidth(res.getDimensionPixelSize(R.dimen.grid_card_width));
             mGridView.setVerticalSpacing(res.getDimensionPixelSize(R.dimen.grid_card_vertical_spacing));
             mVideoAdapter.setListMode(false);
         } else {
             sidePadding = res.getDimensionPixelSize(R.dimen.listview_side_padding);
             mGridView.setNumColumns(1);
-            mGridView.setStretchMode(LIST_STRETCH_MODE);
             mGridView.setVerticalSpacing(0);
             mGridView.setHorizontalSpacing(0);
             mVideoAdapter.setListMode(true);
@@ -333,7 +315,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
                 }
                 return true;
             case R.id.video_list_delete:
-                Snackbar.make(getView(), getString(R.string.file_deleted), Snackbar.LENGTH_LONG)
+                Snackbar.make(getView(), getString(R.string.file_deleted), DELETE_DURATION)
                     .setAction(android.R.string.cancel, mCancelDeleteMediaListener)
                     .show();
                 Message msg = mDeleteHandler.obtainMessage(DELETE_MEDIA, position, 0);
@@ -407,13 +389,8 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
      */
     private Handler mHandler = new VideoListHandler(this);
 
-    public void updateItem() {
-        mVideoAdapter.update(mItemToUpdate);
-        try {
-            mBarrier.await();
-        } catch (InterruptedException e) {
-        } catch (BrokenBarrierException e) {
-        }
+    public void updateItem(MediaWrapper item) {
+        mVideoAdapter.update(item);
     }
     private void focusHelper(boolean idIsEmpty) {
         View parent = getView();
@@ -435,17 +412,16 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         else
             Log.w(TAG, "Can't generate thumbnails, the thumbnailer is missing");
 
-        mVideoAdapter.setNotifyOnChange(true);
-        mVideoAdapter.clear();
-
         if (itemList.size() > 0) {
-            new Thread(new Runnable() {
+            VLCApplication.runBackground(new Runnable() {
                 @Override
                 public void run() {
+                    mVideoAdapter.setNotifyOnChange(false);
+                    mVideoAdapter.clear();
+
                     if (mGroup != null || itemList.size() <= 10) {
                         for (MediaWrapper item : itemList) {
                             if (mGroup == null || item.getTitle().startsWith(mGroup)) {
-                                mVideoAdapter.setNotifyOnChange(false);
                                 mVideoAdapter.add(item);
                                 if (mThumbnailer != null)
                                     mThumbnailer.addJob(item);
@@ -455,16 +431,16 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
                     else {
                         List<MediaGroup> groups = MediaGroup.group(itemList);
                         for (MediaGroup item : groups) {
-                            mVideoAdapter.setNotifyOnChange(false);
                             mVideoAdapter.add(item.getMedia());
                             if (mThumbnailer != null)
-                                mThumbnailer.addJob(item);
+                                mThumbnailer.addJob(item.getMedia());
                         }
                     }
+                    mVideoAdapter.setNotifyOnChange(true);
                     if (mReadyToDisplay)
                         display();
                 }
-            }).start();
+            });
         } else
             focusHelper(true);
         stopRefresh();
@@ -505,22 +481,13 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     }
 
     public void setItemToUpdate(MediaWrapper item) {
-        mItemToUpdate = item;
-        mHandler.sendEmptyMessage(VideoListHandler.UPDATE_ITEM);
+        mHandler.sendMessage(mHandler.obtainMessage(VideoListHandler.UPDATE_ITEM, item));
     }
 
     public void setGroup(String prefix) {
         mGroup = prefix;
     }
 
-    public void await() throws InterruptedException, BrokenBarrierException {
-        mBarrier.await();
-    }
-
-    public void resetBarrier() {
-        mBarrier.reset();
-    }
-
     private final BroadcastReceiver messageReceiverVideoListFragment = new BroadcastReceiver() {
         @Override
         public void onReceive(Context context, Intent intent) {
@@ -543,7 +510,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     @Override
     public void onRefresh() {
         if (getActivity()!=null && !MediaLibrary.getInstance().isWorking())
-            MediaLibrary.getInstance().loadMediaItems(true);
+            MediaLibrary.getInstance().scanMediaItems(true);
     }
 
     @Override
@@ -554,7 +521,6 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
                 public void run() {
                     mViewNomedia.setVisibility(mVideoAdapter.getCount() > 0 ? View.GONE : View.VISIBLE);
                     mReadyToDisplay = true;
-                    mVideoAdapter.setNotifyOnChange(true);
                     mVideoAdapter.sort();
                     mGVFirstVisiblePos = mGridView.getFirstVisiblePosition();
                     mGridView.setSelection(mGVFirstVisiblePos);
@@ -571,11 +537,11 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
     public void deleteMedia(int position){
         final MediaWrapper media = mVideoAdapter.getItem(position);
         final String path = media.getUri().getPath();
-        new Thread(new Runnable() {
+        VLCApplication.runBackground(new Runnable() {
             public void run() {
                 Util.recursiveDelete(VLCApplication.getAppContext(), new File(path));
             }
-        }).start();
+        });
         mMediaLibrary.getMediaItems().remove(media);
         mVideoAdapter.remove(media);
         if (mService != null) {
