@@ -20,12 +20,15 @@
 
 package org.videolan.vlc.util;
 
+import android.annotation.TargetApi;
 import android.content.Context;
 import android.content.res.Resources;
 import android.graphics.Bitmap;
 import android.graphics.BitmapFactory;
 import android.net.Uri;
+import android.os.Build;
 
+import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.vlc.MediaDatabase;
 import org.videolan.vlc.MediaWrapper;
 import org.videolan.vlc.R;
@@ -87,25 +90,33 @@ public class BitmapUtil {
         // mPicture is not null only if passed through
         // the ctor which is deprecated by now.
         Bitmap b = media.getPicture();
-        if(b == null) {
-            BitmapCache cache = BitmapCache.getInstance();
-            Bitmap picture = cache.getBitmapFromMemCache(media.getLocation());
-            if(picture == null) {
-                /* Not in memcache:
-                 * serving the file from the database and
-                 * adding it to the memcache for later use.
-                 */
-                picture = readCoverBitmap(media.getArtworkURL());
-                if (picture == null)
-                    picture = MediaDatabase.getInstance().getPicture(media.getUri());
-                cache.addBitmapToMemCache(media.getLocation(), picture);
-            }
-            return picture;
+        if (b == null) {
+            final BitmapCache cache = BitmapCache.getInstance();
+            return cache.getBitmapFromMemCache(media.getLocation());
         } else {
             return b;
         }
     }
 
+    public static Bitmap fetchPicture(MediaWrapper media) {
+        final BitmapCache cache = BitmapCache.getInstance();
+
+        Bitmap picture = readCoverBitmap(media.getArtworkURL());
+        if (picture == null) {
+            picture = MediaDatabase.getInstance().getPicture(media.getUri());
+        }
+        cache.addBitmapToMemCache(media.getLocation(), picture);
+        return picture;
+    }
+
+    public static Bitmap getPicture(MediaWrapper media) {
+        final Bitmap picture = getPictureFromCache(media);
+        if (picture != null)
+            return picture;
+        else
+            return fetchPicture(media);
+    }
+
     private static Bitmap readCoverBitmap(String path) {
         if (path == null)
             return null;
@@ -139,4 +150,55 @@ public class BitmapUtil {
 
         return cover;
     }
+
+    @TargetApi(Build.VERSION_CODES.KITKAT)
+    static boolean canUseForInBitmap(
+            Bitmap candidate, BitmapFactory.Options targetOptions) {
+
+        if (candidate == null)
+            return false;
+
+        if (AndroidUtil.isKitKatOrLater()) {
+            if (targetOptions.inSampleSize == 0)
+                return false;
+            // From Android 4.4 (KitKat) onward we can re-use if the byte size of
+            // the new bitmap is smaller than the reusable bitmap candidate
+            // allocation byte count.
+            int width = targetOptions.outWidth / targetOptions.inSampleSize;
+            int height = targetOptions.outHeight /  targetOptions.inSampleSize;
+            int byteCount = width * height * getBytesPerPixel(candidate.getConfig());
+            return byteCount <= candidate.getAllocationByteCount();
+        }
+
+        // On earlier versions, the dimensions must match exactly and the inSampleSize must be 1
+        return candidate.getWidth() == targetOptions.outWidth
+                && candidate.getHeight() == targetOptions.outHeight
+                && targetOptions.inSampleSize == 1;
+    }
+
+    /**
+     * A helper function to return the byte usage per pixel of a bitmap based on its configuration.
+     */
+    static int getBytesPerPixel(Bitmap.Config config) {
+        if (config == Bitmap.Config.ARGB_8888) {
+            return 4;
+        } else if (config == Bitmap.Config.RGB_565) {
+            return 2;
+        } else if (config == Bitmap.Config.ARGB_4444) {
+            return 2;
+        } else if (config == Bitmap.Config.ALPHA_8) {
+            return 1;
+        }
+        return 1;
+    }
+
+    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
+    public static void setInBitmap(BitmapFactory.Options options){
+        if (!AndroidUtil.isHoneycombOrLater())
+            return;
+        Bitmap inBitmap = BitmapCache.getInstance().getReusableBitmap(options);
+        if (inBitmap != null) {
+            options.inBitmap = inBitmap;
+        }
+    }
 }
