@@ -21,14 +21,20 @@
 package org.videolan.vlc.gui.tv;
 
 import org.videolan.vlc.R;
+import org.videolan.vlc.gui.tv.browser.BaseTvActivity;
 
-import android.app.Activity;
 import android.os.Bundle;
 
-public class DetailsActivity extends Activity {
+public class DetailsActivity extends BaseTvActivity {
     @Override
     public void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
         setContentView(R.layout.tv_details);
     }
+
+    @Override
+    protected void refresh() {}
+
+    @Override
+    protected void updateList() {}
 }
