@@ -40,6 +40,7 @@ import android.support.v17.leanback.widget.OnItemViewSelectedListener;
 import android.support.v17.leanback.widget.Presenter;
 import android.support.v17.leanback.widget.Row;
 import android.support.v17.leanback.widget.RowPresenter;
+import android.support.v4.util.ArrayMap;
 import android.view.KeyEvent;
 import android.view.View;
 import android.view.View.OnClickListener;
@@ -51,6 +52,7 @@ import org.videolan.vlc.MediaWrapper;
 import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
 import org.videolan.vlc.Thumbnailer;
+import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.PreferencesActivity;
 import org.videolan.vlc.gui.tv.audioplayer.AudioPlayerActivity;
 import org.videolan.vlc.gui.tv.browser.BaseTvActivity;
@@ -62,9 +64,6 @@ import org.videolan.vlc.util.AndroidDevices;
 import org.videolan.vlc.util.VLCInstance;
 
 import java.util.ArrayList;
-import java.util.HashMap;
-import java.util.concurrent.BrokenBarrierException;
-import java.util.concurrent.CyclicBarrier;
 
 public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnItemViewSelectedListener,
         OnItemViewClickedListener, OnClickListener {
@@ -84,24 +83,28 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
 
     protected BrowseFragment mBrowseFragment;
     private ProgressBar mProgressBar;
-    protected final CyclicBarrier mBarrier = new CyclicBarrier(2);
     private static Thumbnailer sThumbnailer;
-    private MediaWrapper mItemToUpdate;
     ArrayObjectAdapter mRowsAdapter = new ArrayObjectAdapter(new ListRowPresenter());
     ArrayObjectAdapter mVideoAdapter;
     ArrayObjectAdapter mCategoriesAdapter;
     ArrayObjectAdapter mNetworkAdapter;
     ArrayObjectAdapter mOtherAdapter;
     View mRootContainer;
-    HashMap<String, Integer> mVideoIndex;
+    final ArrayMap<String, Integer> mVideoIndex = new ArrayMap<String, Integer>();
     Drawable mDefaultBackground;
     Activity mContext;
     private Object mSelectedItem;
+    private AsyncUpdate mUpdateTask;
 
     @Override
     protected void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
-        mMediaLibrary.loadMediaItems(false);
+        if (mMediaLibrary.getMediaItems().isEmpty()) {
+            if (mSettings.getBoolean(PreferencesActivity.AUTO_RESCAN, true))
+                mMediaLibrary.scanMediaItems(false);
+            else
+                mMediaLibrary.loadMedaItems();
+        }
 
         if (!VLCInstance.testCompatibleCPU(this)) {
             finish();
@@ -140,9 +143,8 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
         /*
          * skip browser and show directly Audio Player if a song is playing
          */
-        if (mService.isPlaying()) {
-            startActivity(new Intent(this, AudioPlayerActivity.class));
-            finish();
+        if (service.isPlaying() && !service.canSwitchToVideo()) {
+            updateList();
         }
     }
 
@@ -160,9 +162,6 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
         if (sThumbnailer != null)
             sThumbnailer.setVideoBrowser(this);
 
-        if (mMediaLibrary.isWorking()) //Display UI while MediaLib is scanning
-            updateList();
-
         mBrowseFragment.setBrandColor(getResources().getColor(R.color.orange800));
     }
 
@@ -173,7 +172,6 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
         /* Stop the thumbnailer */
         if (sThumbnailer != null)
             sThumbnailer.setVideoBrowser(null);
-        mBarrier.reset();
     }
 
     @Override
@@ -188,7 +186,7 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
         super.onActivityResult(requestCode, resultCode, data);
         if (requestCode == ACTIVITY_RESULT_PREFERENCES) {
             if (resultCode == PreferencesActivity.RESULT_RESCAN)
-                MediaLibrary.getInstance().loadMediaItems(true);
+                MediaLibrary.getInstance().scanMediaItems(true);
             else if (resultCode == PreferencesActivity.RESULT_RESTART) {
                 Intent intent = getIntent();
                 finish();
@@ -222,16 +220,13 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
         BackgroundManager.getInstance(this).setDrawable(mDefaultBackground);
     }
 
-    public void await() throws InterruptedException, BrokenBarrierException {
-        mBarrier.await();
-    }
-
-    public void resetBarrier() {
-        mBarrier.reset();
-    }
-
     public void updateList() {
-        new AsyncUpdate().execute();
+        if (mUpdateTask == null || mUpdateTask.getStatus() == AsyncTask.Status.FINISHED) {
+            mUpdateTask = new AsyncUpdate();
+            mUpdateTask.execute();
+        } else {
+            mUpdateTask.AskRefresh();
+        }
         checkThumbs();
     }
 
@@ -266,22 +261,15 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
 
     @Override
     public void setItemToUpdate(MediaWrapper item) {
-        mItemToUpdate = item;
-        mHandler.sendEmptyMessage(VideoListHandler.UPDATE_ITEM);
+        mHandler.sendMessage(mHandler.obtainMessage(VideoListHandler.UPDATE_ITEM, item));
     }
 
-    public void updateItem() {
-        if (mVideoAdapter != null && mVideoIndex != null && mItemToUpdate != null) {
-            if (mVideoIndex.containsKey(mItemToUpdate.getLocation())) {
-                mVideoAdapter.notifyArrayItemRangeChanged(mVideoIndex.get(mItemToUpdate.getLocation()), 1);
+    public void updateItem(MediaWrapper item) {
+        if (mVideoAdapter != null && mVideoIndex != null && item != null) {
+            if (mVideoIndex.containsKey(item.getLocation())) {
+                mVideoAdapter.notifyArrayItemRangeChanged(mVideoIndex.get(item.getLocation()).intValue(), 1);
             }
         }
-        try {
-            mBarrier.await();
-        } catch (InterruptedException e) {
-        } catch (BrokenBarrierException e) {
-        }
-
     }
 
     private Handler mHandler = new VideoListHandler(this);
@@ -294,6 +282,10 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
     @Override
     public void onItemClicked(Presenter.ViewHolder itemViewHolder, Object item, RowPresenter.ViewHolder rowViewHolder, Row row) {
         if (row.getId() == HEADER_CATEGORIES) {
+            if (((CardPresenter.SimpleCard)item).getId() == MusicFragment.CATEGORY_NOW_PLAYING){ //NOW PLAYING CARD
+                startActivity(new Intent(this, AudioPlayerActivity.class));
+                return;
+            }
             CardPresenter.SimpleCard card = (CardPresenter.SimpleCard) item;
             Intent intent = new Intent(mContext, VerticalGridActivity.class);
             intent.putExtra(BROWSER_TYPE, HEADER_CATEGORIES);
@@ -315,17 +307,23 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
     }
 
     public class AsyncUpdate extends AsyncTask<Void, Void, Void> {
+        private boolean askRefresh = false;
+        ArrayList<MediaWrapper> videoList;
 
         public AsyncUpdate() {
         }
 
+        public void AskRefresh() { //Ask for refresh while update is ongoing
+            askRefresh = true;
+        }
+
         @Override
         protected void onPreExecute() {
             mRowsAdapter.clear();
             mProgressBar.setVisibility(View.VISIBLE);
 
             //Video Section
-            mVideoIndex = new HashMap<String, Integer>();
+            mVideoIndex.clear();
             mVideoAdapter = new ArrayObjectAdapter(
                     new CardPresenter(mContext));
             final HeaderItem videoHeader = new HeaderItem(HEADER_VIDEO, getString(R.string.video));
@@ -335,6 +333,8 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
             //Music sections
             mCategoriesAdapter = new ArrayObjectAdapter(new CardPresenter(mContext));
             final HeaderItem musicHeader = new HeaderItem(HEADER_CATEGORIES, getString(R.string.audio));
+            if (mService != null && mService.hasMedia() && !mService.canSwitchToVideo())
+                mCategoriesAdapter.add(new CardPresenter.SimpleCard(MusicFragment.CATEGORY_NOW_PLAYING, getString(R.string.music_now_playing), R.drawable.ic_play_circle_big_o));
             mCategoriesAdapter.add(new CardPresenter.SimpleCard(MusicFragment.CATEGORY_ARTISTS, getString(R.string.artists), R.drawable.ic_artist_big));
             mCategoriesAdapter.add(new CardPresenter.SimpleCard(MusicFragment.CATEGORY_ALBUMS, getString(R.string.albums), R.drawable.ic_album_big));
             mCategoriesAdapter.add(new CardPresenter.SimpleCard(MusicFragment.CATEGORY_GENRES, getString(R.string.genres), R.drawable.ic_genre_big));
@@ -365,7 +365,12 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
 
         @Override
         protected Void doInBackground(Void... params) {
-            final ArrayList<MediaWrapper> videoList = mMediaLibrary.getVideoItems();
+            videoList = mMediaLibrary.getVideoItems();
+            return null;
+        }
+
+        @Override
+        protected void onPostExecute(Void result) {
             int size;
             // Update video section
             if (!videoList.isEmpty()) {
@@ -380,23 +385,21 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
                         for (int i = 0; i < total; ++i) {
                             item = videoList.get(i);
                             mVideoAdapter.add(item);
-                            mVideoIndex.put(item.getLocation(), i);
+                            mVideoIndex.put(item.getLocation(), Integer.valueOf(i));
                         }
                     }
                 });
             }
-            return null;
-        }
-
-        @Override
-        protected void onPostExecute(Void result) {
             if (!mMediaLibrary.isWorking())
                 mProgressBar.setVisibility(View.GONE);
+            if (askRefresh) { //in case new event occured while loading view
+                mHandler.sendEmptyMessage(VideoListHandler.MEDIA_ITEMS_UPDATED);
+            }
         }
     }
 
     private void checkThumbs() {
-        new Thread(new Runnable() {
+        VLCApplication.runBackground(new Runnable() {
             @Override
             public void run() {
                 sThumbnailer = new Thumbnailer(mContext, getWindowManager().getDefaultDisplay());
@@ -413,7 +416,7 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
                         sThumbnailer.start((IVideoBrowser) mContext);
                 }
             }
-        }).start();
+        });
     }
 
     public static Thumbnailer getThumbnailer() {
@@ -421,6 +424,6 @@ public class MainTvActivity extends BaseTvActivity implements IVideoBrowser, OnI
     }
 
     protected void refresh() {
-        mMediaLibrary.loadMediaItems(true);
+        mMediaLibrary.scanMediaItems(true);
     }
 }
