@@ -73,7 +73,7 @@ public class RecommendationsService extends IntentService {
     protected void onHandleIntent(Intent intent) {
         if (intent != null && !doRecommendations()) {
             MediaLibrary.getInstance().addUpdateHandler(mHandler);
-            MediaLibrary.getInstance().loadMediaItems();
+            MediaLibrary.getInstance().scanMediaItems();
         }
     }
     private void buildRecommendation(MediaWrapper movie, int id, int priority) {
@@ -96,7 +96,7 @@ public class RecommendationsService extends IntentService {
                         .setOngoing(true)
                         .setColor(mContext.getResources().getColor(R.color.orange800))
                         .setCategory("recommendation")
-                        .setLargeIcon(BitmapUtil.getPictureFromCache(movie))
+                        .setLargeIcon(BitmapUtil.getPicture(movie))
                         .setSmallIcon(R.drawable.icon)
                         .setContentIntent(buildPendingIntent(movie, id))
         ).build();
