@@ -57,6 +57,7 @@ public class AudioPlayerActivity extends BaseTvActivity implements PlaybackServi
     public static final String TAG = "VLC/AudioPlayerActivity";
 
     public static final String MEDIA_LIST = "media_list";
+    public static final String MEDIA_POSITION = "media_position";
 
     private RecyclerView mRecyclerView;
     private PlaylistAdapter mAdapter;
@@ -78,6 +79,7 @@ public class AudioPlayerActivity extends BaseTvActivity implements PlaybackServi
         setContentView(R.layout.tv_audio_player);
 
         mMediaList = getIntent().getParcelableArrayListExtra(MEDIA_LIST);
+        mCurrentlyPlaying = getIntent().getIntExtra(MEDIA_POSITION, 0);
         mRecyclerView = (RecyclerView) findViewById(R.id.playlist);
         mLayoutManager = new LinearLayoutManager(this);
         mRecyclerView.setLayoutManager(mLayoutManager);
@@ -126,7 +128,7 @@ public class AudioPlayerActivity extends BaseTvActivity implements PlaybackServi
         mService.addCallback(this);
         ArrayList<MediaWrapper> medias = (ArrayList<MediaWrapper>) mService.getMedias();
         if (!mMediaList.isEmpty() && !mMediaList.equals(medias)) {
-            mService.load(mMediaList, 0);
+            mService.load(mMediaList, mCurrentlyPlaying);
         } else {
             mMediaList = medias;
             update();
@@ -193,15 +195,17 @@ public class AudioPlayerActivity extends BaseTvActivity implements PlaybackServi
             /*
              * Playback control
              */
-            case KeyEvent.KEYCODE_MEDIA_PLAY_PAUSE:
             case KeyEvent.KEYCODE_MEDIA_PLAY:
             case KeyEvent.KEYCODE_MEDIA_PAUSE:
             case KeyEvent.KEYCODE_SPACE:
                 togglePlayPause();
                 return true;
+            case KeyEvent.KEYCODE_MEDIA_STOP:
+                mService.stop();
+                finish();
+                return true;
             case KeyEvent.KEYCODE_F:
             case KeyEvent.KEYCODE_BUTTON_R1:
-            case KeyEvent.KEYCODE_MEDIA_NEXT:
                 goNext();
                 return true;
             case KeyEvent.KEYCODE_MEDIA_FAST_FORWARD:
@@ -212,7 +216,6 @@ public class AudioPlayerActivity extends BaseTvActivity implements PlaybackServi
                 return true;
             case KeyEvent.KEYCODE_R:
             case KeyEvent.KEYCODE_BUTTON_L1:
-            case KeyEvent.KEYCODE_MEDIA_PREVIOUS:
                 goPrevious();
                 return true;
             /*
@@ -320,15 +323,15 @@ public class AudioPlayerActivity extends BaseTvActivity implements PlaybackServi
     private void updateRepeatMode() {
         if (mService == null)
             return;
-        PlaybackService.RepeatType type = mService.getRepeatType();
-        if (type == PlaybackService.RepeatType.None){
-            mService.setRepeatType(PlaybackService.RepeatType.All);
+        int type = mService.getRepeatType();
+        if (type == PlaybackService.REPEAT_NONE){
+            mService.setRepeatType(PlaybackService.REPEAT_ALL);
             mRepeat.setImageResource(R.drawable.ic_repeat_on);
-        } else if (type == PlaybackService.RepeatType.All) {
-            mService.setRepeatType(PlaybackService.RepeatType.Once);
+        } else if (type == PlaybackService.REPEAT_ALL) {
+            mService.setRepeatType(PlaybackService.REPEAT_ONE);
             mRepeat.setImageResource(R.drawable.ic_repeat_one);
-        } else if (type == PlaybackService.RepeatType.Once) {
-            mService.setRepeatType(PlaybackService.RepeatType.None);
+        } else if (type == PlaybackService.REPEAT_ONE) {
+            mService.setRepeatType(PlaybackService.REPEAT_NONE);
             mRepeat.setImageResource(R.drawable.ic_repeat);
         }
     }
@@ -356,6 +359,11 @@ public class AudioPlayerActivity extends BaseTvActivity implements PlaybackServi
             mService.play();
     }
 
+    private void pause() {
+        if (mService != null && mService.isPlaying())
+            mService.pause();
+    }
+
     private void selectNext() {
         if (mAdapter.getmSelectedItem() >= mAdapter.getItemCount()-1)
             return;
