@@ -31,6 +31,9 @@ import android.support.v17.leanback.widget.OnItemViewClickedListener;
 import android.support.v17.leanback.widget.Presenter;
 import android.support.v17.leanback.widget.Row;
 import android.support.v17.leanback.widget.RowPresenter;
+import android.support.v4.util.ArrayMap;
+import android.support.v4.util.SimpleArrayMap;
+import android.text.TextUtils;
 
 import org.videolan.vlc.MediaLibrary;
 import org.videolan.vlc.MediaWrapper;
@@ -38,12 +41,11 @@ import org.videolan.vlc.R;
 import org.videolan.vlc.gui.tv.MainTvActivity;
 import org.videolan.vlc.gui.audio.MediaComparators;
 import org.videolan.vlc.gui.tv.audioplayer.AudioPlayerActivity;
+import org.videolan.vlc.gui.tv.browser.interfaces.BrowserActivityInterface;
 import org.videolan.vlc.util.WeakHandler;
 
 import java.util.ArrayList;
 import java.util.Collections;
-import java.util.HashMap;
-import java.util.List;
 import java.util.Map;
 
 public class MusicFragment extends MediaLibBrowserFragment {
@@ -55,12 +57,13 @@ public class MusicFragment extends MediaLibBrowserFragment {
     public static final long FILTER_ARTIST = 3;
     public static final long FILTER_GENRE = 4;
 
+    public static final int CATEGORY_NOW_PLAYING = 0;
     public static final int CATEGORY_ARTISTS = 1;
     public static final int CATEGORY_ALBUMS = 2;
     public static final int CATEGORY_GENRES = 3;
     public static final int CATEGORY_SONGS = 4;
 
-    protected Map<String, ListItem> mMediaItemMap;
+    protected SimpleArrayMap<String, ListItem> mMediaItemMap;
     protected ArrayList<ListItem> mMediaItemList;
     private volatile AsyncAudioUpdate mUpdater = null;
 
@@ -83,7 +86,7 @@ public class MusicFragment extends MediaLibBrowserFragment {
 
     public void onResume() {
         super.onResume();
-        if (mAdapter.size() == 0 && mUpdater == null) {
+        if (mUpdater == null) {
             mUpdater = new AsyncAudioUpdate();
             mUpdater.execute();
         }
@@ -105,15 +108,16 @@ public class MusicFragment extends MediaLibBrowserFragment {
 
     public class AsyncAudioUpdate extends AsyncTask<Void, ListItem, String> {
 
+        ArrayList<MediaWrapper> audioList;
         public AsyncAudioUpdate() {}
 
         @Override
         protected void onPreExecute() {
             setTitle(getString(R.string.app_name_full));
             mAdapter.clear();
-            mMediaItemMap = new HashMap<String, ListItem>();
+            mMediaItemMap = new SimpleArrayMap<String, ListItem>();
             mMediaItemList = new ArrayList<ListItem>();
-            ((BrowserActivity)getActivity()).showProgress(true);
+            ((BrowserActivityInterface)getActivity()).showProgress(true);
         }
 
         @Override
@@ -121,7 +125,7 @@ public class MusicFragment extends MediaLibBrowserFragment {
             String title;
             ListItem item;
 
-            List<MediaWrapper> audioList = MediaLibrary.getInstance().getAudioItems();
+            audioList = MediaLibrary.getInstance().getAudioItems();
             if (CATEGORY_ARTISTS == mCategory){
                 Collections.sort(audioList, MediaComparators.byArtist);
                 title = getString(R.string.artists);
@@ -178,7 +182,7 @@ public class MusicFragment extends MediaLibBrowserFragment {
 
         @Override
         protected void onPostExecute(String title) {
-            ((BrowserActivity)getActivity()).showProgress(false);
+            ((BrowserActivityInterface)getActivity()).showProgress(false);
             setTitle(title);
             setOnItemViewClickedListener(new OnItemViewClickedListener() {
                 @Override
@@ -199,10 +203,22 @@ public class MusicFragment extends MediaLibBrowserFragment {
                         intent.putExtra(MEDIA_SECTION, FILTER_GENRE);
                         intent.putExtra(AUDIO_FILTER, listItem.mediaList.get(0).getGenre());
                     } else {
-                        if (CATEGORY_ALBUMS == mCategory)
-                            Collections.sort(listItem.mediaList, MediaComparators.byTrackNumber);
                         intent = new Intent(mContext, AudioPlayerActivity.class);
-                        intent.putExtra(AudioPlayerActivity.MEDIA_LIST, listItem.mediaList);
+                        if (CATEGORY_ALBUMS == mCategory) {
+                            Collections.sort(listItem.mediaList, MediaComparators.byTrackNumber);
+                            intent.putExtra(AudioPlayerActivity.MEDIA_LIST, listItem.mediaList);
+                        } else {
+                            int position = 0;
+                            String location = listItem.mediaList.get(0).getLocation();
+                            for (int i = 0 ; i< audioList.size() ; ++i) {
+                                if (TextUtils.equals(location, audioList.get(i).getLocation())) {
+                                    position = i;
+                                    break;
+                                }
+                            }
+                            intent.putExtra(AudioPlayerActivity.MEDIA_LIST, audioList);
+                            intent.putExtra(AudioPlayerActivity.MEDIA_POSITION, position);
+                        }
                     }
                     startActivity(intent);
                 }
@@ -224,14 +240,14 @@ public class MusicFragment extends MediaLibBrowserFragment {
         }
     }
 
-    public ListItem add(String title, String subTitle, MediaWrapper MediaWrapper) {
+    public ListItem add(String title, String subTitle, MediaWrapper mediaWrapper) {
         if(title == null) return null;
         title = title.trim();
         if(subTitle != null) subTitle = subTitle.trim();
         if (mMediaItemMap.containsKey(title))
-            mMediaItemMap.get(title).mediaList.add(MediaWrapper);
+            mMediaItemMap.get(title).mediaList.add(mediaWrapper);
         else {
-            ListItem item = new ListItem(title, subTitle, MediaWrapper);
+            ListItem item = new ListItem(title, subTitle, mediaWrapper);
             mMediaItemMap.put(title, item);
             mMediaItemList.add(item);
             return item;
@@ -240,7 +256,7 @@ public class MusicFragment extends MediaLibBrowserFragment {
     }
 
     @Override
-    protected void updateList() {
+    public void updateList() {
         if (mUpdater == null) {
             mUpdater = new AsyncAudioUpdate();
             mUpdater.execute();
@@ -260,7 +276,8 @@ public class MusicFragment extends MediaLibBrowserFragment {
             super.handleMessage(msg);
             switch (msg.what){
                 case MediaLibrary.MEDIA_ITEMS_UPDATED:
-                    getOwner().updateList();
+                    if (getOwner() != null)
+                        getOwner().updateList();
                     break;
             }
         }
